@@ -5,7 +5,7 @@ Object::Base - Multi-threaded base class to establish a class deriving relations
 
 =head1 VERSION
 
-version 1.03
+version 1.04
 
 =head1 ABSTRACT
 
@@ -20,7 +20,7 @@ Multi-threaded base class to establish a class deriving relationship with parent
 
 =head1 DESCRIPTION
 
-Object::Base provides blessed and thread-shared(with :shared attribute) object with in B<new> method. B<new> method
+Object::Base provides blessed and thread-shared(with :shared feature) object with in B<new> method. B<new> method
 can be used as a constructor and overridable in derived classes. B<new()> should be called in derived class
 constructors to create and bless self-object.
 
@@ -32,8 +32,10 @@ Import parameters of Object::Base, define parent classes of derived class.
 If none of parent classes derived from Object::Base or any parent isn't defined, Object::Base is automatically added
 in parent classes.
 
+=head2 Attributes
+
 Attributes define read-write accessors binded value of same named key in objects own hash if attribute names is
-valid subroutine identifiers. Otherwise, attribute is class feature to get new features into class.
+valid subroutine identifiers. Otherwise, attribute defines B<feature> to get new features into class.
 
 Attributes;
 
@@ -61,11 +63,83 @@ Thread-Safe
 
 =back
 
-Examples;
+=head3 Modifiers
+
+Attributes can have their own modifiers in hash reference at definition.
+
+=head4 default
+
+getter method of default value of attribute, otherwise value is default value
+
+	attributes
+		'attr1' => {
+			'default' => sub {
+				my ($self, $attr) = @_;
+				return "default value of $attr";
+			},
+		},
+		'attr2' => {
+			'default' => "default value of attr2",
+		};
+
+=head4 getter
+
+getter method of attribute
+
+	my $attr1_val;
+	attributes
+		'attr1' => {
+			'getter' => sub {
+				my ($self, $attr, $current_value) = @_;
+				return $attr1_val;
+			},
+		};
+
+=head4 setter
+
+setter method of attribute
+
+	my $attr1_val;
+	attributes
+		'attr1' => {
+			'setter' => sub {
+				my ($self, $attr, $current_value, $new_value) = @_;
+				$attr1_val = $new_value;
+			},
+		};
+
+=head3 Features
+
+=head4 :shared
+
+Class will be craated as thread-shared.
+
+=head4 :lazy
+
+Attribute default value will be initialized at first fetching or storing.
+
+=head2 Examples
 
 	package Foo;
 	use Object::Base;
-	attributes ':shared', 'attr1', 'attr2';
+	my $attr3_def = 6;
+	my $attr3_val;
+	attributes ':shared', 'attr1', 'attr2', ':lazy',
+		'attr3' => {
+			'default' => sub {
+				my ($self, $attr) = @_;
+				$attr3_val = $attr3_def;
+				return $attr3_def;
+			},
+			'getter' => sub {
+				my ($self, $attr, $current_value) = @_;
+				return $attr3_val+1;
+			},
+			'setter' => sub {
+				my ($self, $attr, $current_value, $new_value) = @_;
+				$attr3_val = $new_value-1;
+			},
+		};
 	
 	package Bar;
 	use Object::Base 'Foo';
@@ -89,6 +163,9 @@ Examples;
 	# special attribute ':shared'
 	print "\$foo is ", is_shared($foo)? "shared": "not shared", "\n";
 	
+	# attributes can have modifiers: default, getter, setter
+	print "attr3 value ", $foo->attr3, " and stored as $attr3_val", "\n"; # prints 'attr3 value 7 and stored as 6'
+	
 	# object of derived class Bar
 	my $bar = Bar->new();
 	
@@ -108,6 +185,12 @@ Examples;
 	eval { $foo->attr2 = { key1 => 'val1' } }; print "Eval: $@"; # prints error 'Eval: Invalid value for shared scalar at ...'
 	$foo->attr2({ key2 => 'val2' }); # uses shared_clone assigning ref value
 	print $foo->attr2->{key2}, "\n"; # prints 'val2'
+	
+	# attributes in thread
+	my $thr1 = threads->create(sub { $foo->attr1 = 5; $bar->attr1 = 5; });
+	my $thr2 = threads->create(sub { sleep 1; print "\$foo is shared and attr1: ", $foo->attr1, ", \$bar is not shared and attr1: ", $bar->attr1, "\n"; });
+	$thr1->join();
+	$thr2->join();
 
 =cut
 BEGIN
@@ -134,7 +217,7 @@ use warnings;
 BEGIN
 {
 	require 5.008;
-	$Object::Base::VERSION = '1.03';
+	$Object::Base::VERSION = '1.04';
 	$Object::Base::ISA = ();
 }
 
@@ -228,7 +311,7 @@ sub $_ :lvalue
 	my \@args = \@_;
 	if (\@args >= 1)
 	{
-		unless (ref(\$args[0]) and \$${caller}::${context}{':shared'})
+		unless (ref(\$args[0]) and is_shared(%{\$self}))
 		{
 			\$self->{"\Q$_\E"} = \$args[0];
 		} else
@@ -288,33 +371,35 @@ BEGIN
 sub TIEHASH
 {
 	my $class = shift;
-	my $self = [{}, @_];
-	$self = shared_clone($self) if ${"$self->[1]::${context}"}{":shared"};
-	bless $self, $class;
-	unless (${"$self->[1]::${context}"}{":lazy"})
+	my $self = [{}, @_, {}];
+	if (${"$self->[1]::${context}"}{":shared"})
 	{
-		for (grep /^[^\W\d]\w*\z/s, keys(%{"$self->[1]::${context}"}))
-		{
-			$self->def($_);
-		}
+		$self->[0] = shared_clone($self->[0]);
+		$self->[$#{$self}] = shared_clone($self->[$#{$self}]);
+	}
+	bless $self, $class;
+	for (grep /^[^\W\d]\w*\z/s, keys(%{"$self->[1]::${context}"}))
+	{
+		$self->[0]->{$_} = undef;
+		$self->def($_) unless ${"$self->[1]::${context}"}{":lazy"};
 	}
 	$self;
 }
 
 sub STORE
 {
-	lock($_[0]) if ${"$_[0]->[1]::${context}"}{":shared"};
+	lock(%{$_[0]->[0]}) if is_shared(%{$_[0]->[0]});
 	my $self = shift;
 	my ($key, $value) = @_;
 	return unless $key =~ /^[^\W\d]\w*\z/s;
-	$self->def($key) unless exists($self->[0]->{$key});
+	$self->def($key);
 	my $attr = ${"$self->[1]::${context}"}{$key};
 	if (ref($attr) eq 'HASH' and exists($attr->{"setter"}))
 	{
 		my $setter = $attr->{"setter"};
 		if (ref($setter) eq 'CODE')
 		{
-			$setter->(${$self->[2]}, $key, $value);
+			$setter->(${$self->[2]}, $key, $self->[0]->{$key}, $value);
 		}
 	}
 	$self->[0]->{$key} = $value;
@@ -322,49 +407,53 @@ sub STORE
 
 sub FETCH
 {
-	lock($_[0]) if ${"$_[0]->[1]::${context}"}{":shared"};
+	lock(%{$_[0]->[0]}) if is_shared(%{$_[0]->[0]});
 	my $self = shift;
-	my ($key, $value) = @_;
+	my ($key) = @_;
 	return unless $key =~ /^[^\W\d]\w*\z/s;
-	$self->def($key) unless exists($self->[0]->{$key});
+	$self->def($key);
 	my $attr = ${"$self->[1]::${context}"}{$key};
 	if (ref($attr) eq 'HASH' and exists($attr->{"getter"}))
 	{
 		my $getter = $attr->{"getter"};
 		if (ref($getter) eq 'CODE')
 		{
-			$self->[0]->{$key} = $getter->(${$self->[2]}, $key, $value, $self->[0]->{$key});
+			$self->[0]->{$key} = $getter->(${$self->[2]}, $key, $self->[0]->{$key});
 		}
 	}
 	$self->[0]->{$key};
 }
 
-sub FIRSTKEY { lock($_[0]) if ${"$_[0]->[1]::${context}"}{":shared"}; my $a = scalar keys %{$_[0][0]}; each %{$_[0][0]} }
-sub NEXTKEY  { lock($_[0]) if ${"$_[0]->[1]::${context}"}{":shared"}; each %{$_[0][0]} }
-sub EXISTS   { lock($_[0]) if ${"$_[0]->[1]::${context}"}{":shared"}; exists $_[0][0]->{$_[1]} }
-sub DELETE   { lock($_[0]) if ${"$_[0]->[1]::${context}"}{":shared"}; delete $_[0][0]->{$_[1]} }
-sub CLEAR    { lock($_[0]) if ${"$_[0]->[1]::${context}"}{":shared"}; %{$_[0][0]} = () }
-sub SCALAR   { lock($_[0]) if ${"$_[0]->[1]::${context}"}{":shared"}; scalar %{$_[0][0]} }
+sub FIRSTKEY { lock(%{$_[0]->[0]}) if is_shared(%{$_[0]->[0]}); my $a = scalar keys %{$_[0][0]}; each %{$_[0][0]} }
+sub NEXTKEY  { lock(%{$_[0]->[0]}) if is_shared(%{$_[0]->[0]}); each %{$_[0][0]} }
+sub EXISTS   { lock(%{$_[0]->[0]}) if is_shared(%{$_[0]->[0]}); exists $_[0][0]->{$_[1]} }
+sub DELETE   { lock(%{$_[0]->[0]}) if is_shared(%{$_[0]->[0]}); delete $_[0][$#{$_[0]}]->{$_[1]}; delete $_[0][0]->{$_[1]} }
+sub CLEAR    { lock(%{$_[0]->[0]}) if is_shared(%{$_[0]->[0]}); %{$_[0][$#{$_[0]}]} = (); %{$_[0][0]} = () }
+sub SCALAR   { lock(%{$_[0]->[0]}) if is_shared(%{$_[0]->[0]}); scalar %{$_[0][0]} }
 
 sub def
 {
-	lock($_[0]) if ${"$_[0]->[1]::${context}"}{":shared"};
+	lock(%{$_[0]->[0]}) if is_shared(%{$_[0]->[0]});
 	my $self = shift;
 	my ($key) = @_;
 	return unless $key =~ /^[^\W\d]\w*\z/s;
-	my $attr = ${"$self->[1]::${context}"}{$key};
-	if (ref($attr) eq 'HASH' and exists($attr->{"default"}))
+	unless (exists($self->[$#{$self}]->{$key}))
 	{
-		my $default = $attr->{"default"};
-		if (ref($default) eq 'CODE')
+		my $attr = ${"$self->[1]::${context}"}{$key};
+		if (ref($attr) eq 'HASH' and exists($attr->{"default"}))
 		{
-			return $self->[0]{$key} = $default->(${$self->[2]}, $key);
-		} else
-		{
-			return $self->[0]{$key} = $default;
+			my $default = $attr->{"default"};
+			if (ref($default) eq 'CODE')
+			{
+				$self->[$#{$self}]->{$key} = $default->(${$self->[2]}, $key);
+			} else
+			{
+				$self->[$#{$self}]->{$key} = $default;
+			}
+			$self->[0]{$key} = $self->[$#{$self}]->{$key};
 		}
 	}
-	return;
+	return $self->[$#{$self}]->{$key};
 }
 
 
@@ -397,6 +486,10 @@ threads
 
 threads::shared
 
+=item *
+
+forks
+
 =back
 
 =head1 REPOSITORY
